<?php
/*
 * 파일명: simple-shorturl.php
 * 위치: /wp-content/plugins/simple-shorturl/
 * 기능: 포스트에 단축 URL을 생성하고 필요시에만 본문 하단에 표시하는 워드프레스 플러그인
 * 작성일: 2025-03-30
 * 수정일: 2025-04-17
 * 
 * Plugin Name: Simple Short URL
 * Plugin URI: https://url.kr/p/wp-plugin/simple-shorturl/
 * Description: 포스트에 4자리 단축 URL을 생성하고 필요시에만 본문 하단에 표시합니다.
 * Version: 2.0
 * Author: 호텔천사
 * Author URI: https://webworks.co.kr
 * License: GPL2
 */

// 직접 접근 방지
if (!defined('ABSPATH')) {
    exit;
}

// ===================================
// 플러그인 초기화 및 설정
// ===================================

/**
 * 플러그인 활성화 시 실행되는 함수
 */
function ssu_activate() {
    // 기본 설정값 추가
    add_option('ssu_enabled', '1');
    
    // Rewrite 규칙 추가 및 플러시
    ssu_add_rewrite_rule();
    flush_rewrite_rules();
}
register_activation_hook(__FILE__, 'ssu_activate');

/**
 * 플러그인 비활성화 시 실행되는 함수
 */
function ssu_deactivate() {
    flush_rewrite_rules();
}
register_deactivation_hook(__FILE__, 'ssu_deactivate');

/**
 * 플러그인 제거 시 실행되는 함수
 */
function ssu_uninstall() {
    // 옵션 삭제
    delete_option('ssu_enabled');
    
    // 모든 포스트의 단축 URL 메타데이터 삭제
    $args = array(
        'post_type' => 'post',
        'posts_per_page' => -1,
        'meta_query' => array(
            array(
                'key' => '_ssu_short_code',
                'compare' => 'EXISTS',
            ),
        ),
    );
    $query = new WP_Query($args);
    
    if ($query->have_posts()) {
        while ($query->have_posts()) {
            $query->the_post();
            delete_post_meta(get_the_ID(), '_ssu_short_code');
        }
    }
    wp_reset_postdata();
}
register_uninstall_hook(__FILE__, 'ssu_uninstall');

// ===================================
// Rewrite 규칙 및 URL 처리
// ===================================

/**
 * Rewrite 규칙 추가 - 루트에 4자리 코드 매칭 (예: site.com/abcd)
 */
function ssu_add_rewrite_rule() {
    add_rewrite_rule('^([a-z0-9]{4})/?$', 'index.php?short_code=$matches[1]', 'top');
}
add_action('init', 'ssu_add_rewrite_rule');

/**
 * 사용자 정의 쿼리 변수 등록
 */
function ssu_query_vars($query_vars) {
    $query_vars[] = 'short_code';
    return $query_vars;
}
add_filter('query_vars', 'ssu_query_vars');

/**
 * 301 리다이렉트 수행: 단축 코드에 해당하는 포스트로 리다이렉트
 */
function ssu_template_redirect() {
    global $wp_query;
    
    // 설정이 활성화되어 있는지 확인
    if (get_option('ssu_enabled', '1') !== '1') {
        return;
    }
    
    // short_code 파라미터가 있는지 확인
    if (isset($wp_query->query_vars['short_code']) && !empty($wp_query->query_vars['short_code'])) {
        $code = sanitize_text_field($wp_query->query_vars['short_code']);
        
        // 해당 코드를 가진 포스트 찾기
        $args = array(
            'post_type' => 'post',
            'meta_query' => array(
                array(
                    'key' => '_ssu_short_code',
                    'value' => $code,
                    'compare' => '='
                )
            ),
            'posts_per_page' => 1,
            'fields' => 'ids'
        );
        $posts = get_posts($args);
        
        if (!empty($posts)) {
            $post_url = get_permalink($posts[0]);
            if ($post_url) {
                wp_redirect($post_url, 301);
                exit;
            }
        }
        
        // 코드에 해당하는 포스트를 찾지 못한 경우 404 처리
        $wp_query->set_404();
        status_header(404);
        get_template_part(404);
        exit;
    }
}
add_action('template_redirect', 'ssu_template_redirect');

// ===================================
// 단축 URL 생성 및 관리
// ===================================

/**
 * 4자리 랜덤 단축코드 생성
 * 
 * @return string 생성된 4자리 단축코드
 */
function ssu_generate_short_code() {
    $characters = 'abcdefghijklmnopqrstuvwxyz0123456789';
    $characters_length = strlen($characters);
    
    // 고유한 코드 생성 (충돌이 없을 때까지 반복)
    do {
        $code = '';
        for ($i = 0; $i < 4; $i++) {
            $code .= $characters[rand(0, $characters_length - 1)];
        }
        
        // 이미 사용 중인 코드인지 확인
        $existing = get_posts(array(
            'post_type' => 'post',
            'meta_query' => array(
                array(
                    'key' => '_ssu_short_code',
                    'value' => $code,
                    'compare' => '='
                )
            ),
            'posts_per_page' => 1,
            'fields' => 'ids'
        ));
    } while (!empty($existing));
    
    return $code;
}

/**
 * AJAX 핸들러: 단축 URL 생성 및 표시
 */
function ssu_generate_shorturl_ajax() {
    // 보안 검증
    check_ajax_referer('ssu_generate_shorturl', 'nonce');
    
    // 포스트 ID 가져오기
    $post_id = isset($_POST['post_id']) ? intval($_POST['post_id']) : 0;
    
    if ($post_id <= 0) {
        wp_send_json_error('유효하지 않은 포스트 ID입니다.');
        return;
    }
    
    // 포스트 존재 여부 확인
    $post = get_post($post_id);
    if (!$post || $post->post_type !== 'post' || $post->post_status !== 'publish') {
        wp_send_json_error('유효한 발행된 포스트가 아닙니다.');
        return;
    }
    
    // 단축 URL 가져오거나 생성
    $short_code = get_post_meta($post_id, '_ssu_short_code', true);
    if (empty($short_code)) {
        $short_code = ssu_generate_short_code();
        update_post_meta($post_id, '_ssu_short_code', $short_code);
    }
    
    $short_url = home_url('/' . $short_code);
    
    // HTML 응답 생성
    $html = '
    <div class="short-url-result" style="margin-top:10px; text-align:center;">
        <div style="display:flex; flex-wrap:wrap; align-items:center; justify-content:center;">
            <span style="font-weight:500; margin-right:5px; white-space:nowrap;">단축 URL:</span>
            <a href="' . esc_url($short_url) . '" target="_blank" class="short-url-text" style="font-size:16px; font-weight:bold; text-decoration:none; color:#2196F3; overflow:hidden; text-overflow:ellipsis;">' . esc_html($short_url) . '</a>
            <button class="copy-short-url" style="margin-left:10px; padding:5px 10px; border:none; background-color:#673AB7; color:white; border-radius:4px; cursor:pointer; white-space:nowrap;">
                Copy
            </button>
        </div>
    </div>
    ';
    
    wp_send_json_success(array(
        'html' => $html,
        'shortUrl' => $short_url
    ));
}
add_action('wp_ajax_ssu_generate_shorturl', 'ssu_generate_shorturl_ajax');
add_action('wp_ajax_nopriv_ssu_generate_shorturl', 'ssu_generate_shorturl_ajax');

// ===================================
// 관리자 설정 페이지
// ===================================

/**
 * 관리자 메뉴에 설정 페이지 추가
 */
function ssu_add_admin_menu() {
    add_options_page(
        'Short URL 설정',
        'Short URL',
        'manage_options',
        'simple_shorturl',
        'ssu_options_page'
    );
}
add_action('admin_menu', 'ssu_add_admin_menu');

/**
 * 설정 페이지 레지스터
 */
function ssu_settings_init() {
    register_setting('ssu_option_group', 'ssu_enabled');
}
add_action('admin_init', 'ssu_settings_init');

/**
 * 설정 페이지 HTML
 */
function ssu_options_page() {
    // 부트스트랩 아이콘 추가
    wp_enqueue_style('bootstrap-icons', 'https://cdn.jsdelivr.net/npm/bootstrap-icons@1.11.1/font/bootstrap-icons.css');
    ?>
    <div class="wrap">
        <h1>단축 URL 설정</h1>
        <form action="options.php" method="post">
            <?php settings_fields('ssu_option_group'); ?>
            <?php do_settings_sections('ssu_option_group'); ?>
            <table class="form-table">
                <tr valign="top">
                    <th scope="row">단축 URL 기능 사용</th>
                    <td>
                        <div class="input-group" style="display: flex; align-items: center;">
                            <span style="margin-right: 10px;">
                                <i class="bi bi-link-45deg" style="color: #0073aa; font-size: 1.2em;"></i>
                            </span>
                            <input type="checkbox" name="ssu_enabled" value="1" <?php checked(get_option('ssu_enabled', '1'), '1'); ?> />
                            <span style="margin-left: 10px;">외부 캠페인 및 소셜 미디어 공유용 단축 URL 기능을 사용합니다.</span>
                        </div>
                    </td>
                </tr>
            </table>
            <?php submit_button(); ?>
        </form>
    </div>
    <?php
}

// ===================================
// 프론트엔드 표시
// ===================================

/**
 * 본문 하단에 단축 URL 생성 버튼 표시
 * 
 * @param string $content 포스트 내용
 * @return string 수정된 포스트 내용
 */
function ssu_add_shorturl_to_content($content) {
    if (is_singular('post') && in_the_loop() && is_main_query()) {
        $enabled = get_option('ssu_enabled', '1');
        if ($enabled) {
            global $post;
            $unique_id = 'short-url-' . $post->ID;
            
            // 버튼 추가
            $html = '
            <div id="' . esc_attr($unique_id) . '" class="short-url-container" style="border:1px solid #ccc; padding:10px; margin:20px auto; text-align:center; max-width:900px;">
                <button id="generate-short-url-' . esc_attr($post->ID) . '" class="generate-short-url-btn" style="padding:8px 15px; background-color:#4CAF50; color:white; border:none; border-radius:4px; cursor:pointer; font-weight:500;">
                    <i class="bi bi-link-45deg" style="margin-right:5px;"></i> 단축주소 생성하기
                </button>
                <div id="short-url-result-' . esc_attr($post->ID) . '"></div>
            </div>
            
            <script>
            (function(){
                document.addEventListener("DOMContentLoaded", function() {
                    var container = document.getElementById("' . esc_js($unique_id) . '");
                    if(container){
                        var generateBtn = document.getElementById("generate-short-url-' . esc_js($post->ID) . '");
                        var resultContainer = document.getElementById("short-url-result-' . esc_js($post->ID) . '");
                        
                        generateBtn.addEventListener("click", function(){
                            // 버튼 비활성화 및 로딩 표시
                            generateBtn.disabled = true;
                            generateBtn.innerHTML = "<i class=\'bi bi-arrow-repeat\'></i> 생성 중...";
                            
                            // AJAX 요청
                            var xhr = new XMLHttpRequest();
                            xhr.open("POST", "' . esc_js(admin_url('admin-ajax.php')) . '", true);
                            xhr.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
                            xhr.onreadystatechange = function() {
                                if (xhr.readyState === 4) {
                                    // 버튼 상태 복원
                                    generateBtn.disabled = false;
                                    generateBtn.innerHTML = "<i class=\'bi bi-link-45deg\'></i> 단축주소 생성하기";
                                    
                                    if (xhr.status === 200) {
                                        try {
                                            var response = JSON.parse(xhr.responseText);
                                            if (response.success) {
                                                // 결과 표시
                                                resultContainer.innerHTML = response.data.html;
                                                
                                                // 복사 버튼 이벤트 추가
                                                var copyBtn = resultContainer.querySelector(".copy-short-url");
                                                var urlText = response.data.shortUrl;
                                                var originalBtnText = copyBtn.innerText;
                                                
                                                copyBtn.addEventListener("click", function(){
                                                    if(navigator.clipboard){
                                                        navigator.clipboard.writeText(urlText).then(function(){
                                                            copyBtn.innerText = "Copied!";
                                                            setTimeout(function() {
                                                                copyBtn.innerText = originalBtnText;
                                                            }, 1500);
                                                        }, function(err){
                                                            alert("복사 실패: " + err);
                                                        });
                                                    } else {
                                                        var textarea = document.createElement("textarea");
                                                        textarea.value = urlText;
                                                        document.body.appendChild(textarea);
                                                        textarea.select();
                                                        try {
                                                            document.execCommand("copy");
                                                            copyBtn.innerText = "Copied!";
                                                            setTimeout(function() {
                                                                copyBtn.innerText = originalBtnText;
                                                            }, 1500);
                                                        } catch (err) {
                                                            alert("복사 실패: " + err);
                                                        }
                                                        document.body.removeChild(textarea);
                                                    }
                                                });
                                                
                                                // 버튼 숨기기
                                                generateBtn.style.display = "none";
                                                
                                                // 반응형 스타일 조정
                                                function adjustStyles() {
                                                    var containerWidth = container.offsetWidth;
                                                    var resultDiv = resultContainer.querySelector(".short-url-result");
                                                    
                                                    if (containerWidth < 480) {
                                                        // 모바일 화면에서는 세로 정렬 (가운데 정렬 유지)
                                                        resultDiv.querySelector("div").style.flexDirection = "column";
                                                        resultDiv.querySelector("div").style.alignItems = "center";
                                                        resultDiv.querySelector(".short-url-text").style.margin = "5px 0";
                                                        resultDiv.querySelector(".copy-short-url").style.marginLeft = "0";
                                                        resultDiv.querySelector(".copy-short-url").style.marginTop = "5px";
                                                    } else {
                                                        // 데스크톱 화면에서는 가로 정렬 (가운데 정렬 유지)
                                                        resultDiv.querySelector("div").style.flexDirection = "row";
                                                        resultDiv.querySelector(".short-url-text").style.margin = "0";
                                                        resultDiv.querySelector(".copy-short-url").style.marginLeft = "10px";
                                                        resultDiv.querySelector(".copy-short-url").style.marginTop = "0";
                                                    }
                                                }
                                                
                                                // 스타일 조정 실행
                                                adjustStyles();
                                                window.addEventListener("resize", adjustStyles);
                                            } else {
                                                resultContainer.innerHTML = "<div class=\'error\' style=\'color:red;margin-top:10px;\'>오류: " + (response.data || "단축 URL 생성 실패") + "</div>";
                                            }
                                        } catch (e) {
                                            resultContainer.innerHTML = "<div class=\'error\' style=\'color:red;margin-top:10px;\'>오류: 서버 응답 처리 실패</div>";
                                        }
                                    } else {
                                        resultContainer.innerHTML = "<div class=\'error\' style=\'color:red;margin-top:10px;\'>오류: 서버 연결 실패</div>";
                                    }
                                }
                            };
                            xhr.send("action=ssu_generate_shorturl&post_id=' . esc_js($post->ID) . '&nonce=' . esc_js(wp_create_nonce('ssu_generate_shorturl')) . '");
                        });
                    }
                });
            })();
            </script>
            ';
            
            // 부트스트랩 아이콘 스타일시트 추가
            wp_enqueue_style('bootstrap-icons', 'https://cdn.jsdelivr.net/npm/bootstrap-icons@1.11.1/font/bootstrap-icons.css');
            
            return $content . $html;
        }
    }
    return $content;
}

add_filter('the_content', 'ssu_add_shorturl_to_content');